@@ -3,7 +3,7 @@ use std::fmt::{Display, Write};
 use std::path::{Path, PathBuf};
 use std::{env, fs};
 
-use anyhow::{Error, Result};
+use anyhow::Result;
 #[cfg(feature = "manifest")]
 use cargo_toml::{Manifest, Product};
 use log::*;
@@ -224,8 +224,7 @@ build-std-features = ["panic_immediate_abort"]
         cargo_toml
             .lib
             .as_ref()
-            .map(|lib| lib.name.clone())
-            .flatten()
+            .and_then(|lib| lib.name.clone())
             .unwrap_or_else(|| {
                 cargo_toml
                     .package
@@ -285,10 +284,8 @@ build-std-features = ["panic_immediate_abort"]
         self.scan_config_toml(|value| {
             value
                 .get("build")
-                .map(|table| table.get("target"))
-                .flatten()
-                .map(|value| value.as_str())
-                .flatten()
+                .and_then(|table| table.get("target"))
+                .and_then(|value| value.as_str())
                 .map(|str| str.to_owned())
         })
     }
@@ -330,7 +327,7 @@ pub fn set_rustc_cfg(key: impl Display, value: impl AsRef<str>) {
         println!(
             "cargo:rustc-cfg={}=\"{}\"",
             key,
-            value.as_ref().replace("\"", "\\\"")
+            value.as_ref().replace('\"', "\\\"")
         );
     }
 }
@@ -355,24 +352,37 @@ pub fn out_dir() -> PathBuf {
         .into()
 }
 
-pub trait IntoWarning {
-    type T;
-
-    /// Print a cargo warning for this error.
-    fn into_warning(self) -> Self::T;
+/// Extension trait for turning [`Display`]able values into cargo warnings.
+pub trait IntoWarning<R> {
+    /// Print as a cargo warning.
+    ///
+    /// This will `{:#}`-print all lines using `println!("cargo:warning={}", line)` where
+    /// the first line's `Error:` prefix is removed and trimmed.
+    fn into_warning(self) -> R;
 }
 
-impl IntoWarning for Error {
-    type T = ();
+impl<E> IntoWarning<()> for E
+where
+    E: Display,
+{
     fn into_warning(self) {
-        for line in format!("{:#}", self.context("error turned into warning")).lines() {
+        let fmt = format!("{:#}", self);
+        let fmt = fmt.strip_prefix("Error:").unwrap_or(&fmt).trim_start();
+        let mut lines = fmt.lines();
+
+        let line = lines.next().unwrap_or("(empty)");
+        cargo::print_warning(line);
+
+        for line in lines {
             cargo::print_warning(line);
         }
     }
 }
 
-impl<V> IntoWarning for Result<V, Error> {
-    type T = Option<V>;
+impl<V, E> IntoWarning<Option<V>> for Result<V, E>
+where
+    E: IntoWarning<()>,
+{
     fn into_warning(self) -> Option<V> {
         match self {
             Ok(v) => Some(v),
@@ -393,6 +403,14 @@ pub fn workspace_dir() -> Option<PathBuf> {
     // We pop the path to the out dir 6 times to get to the workspace root so the
     // directory containing the `target` (build) directory. The directory containing the
     // `target` directory is always the workspace root.
-    // FIXME: when $HOST == $TARGET this is potentially one less
-    Some(PathBuf::from(env::var_os("OUT_DIR")?).pop_times(6))
+
+    // We have to pop one less if `$HOST == $TARGET` because then cargo will compile
+    // directly into the `debug` or `release` directory instead of having that directory
+    // inside of a `<target>` directory.
+    let pop_count = if env::var_os("HOST")? == env::var_os("TARGET")? {
+        5
+    } else {
+        6
+    };
+    Some(PathBuf::from(env::var_os("OUT_DIR")?).pop_times(pop_count))
 }
