@@ -203,6 +203,7 @@ impl Pio {
         pio_installer.check().map(|pio| pio.log_level(log_level))
     }
 
+    #[must_use]
     pub fn log_level(mut self, log_level: LogLevel) -> Self {
         self.log_level = log_level;
 
@@ -524,6 +525,7 @@ impl PioInstaller {
 }
 
 #[derive(Clone, Debug)]
+#[must_use]
 pub struct Resolver {
     pio: Pio,
     params: ResolutionParams,
@@ -975,7 +977,7 @@ impl Resolver {
                         Some(s2)
                     }
                 })
-                .unwrap_or_else(Vec::new);
+                .unwrap_or_default();
 
             if platforms.is_empty() {
                 bail!("Cannot select a platform: configured frameworks [{}] do not have a common platform", params.frameworks.join(", "));
